<?php

namespace QUI\CmsImport;

abstract class AbstractImportProvider implements ImportProviderInterface
{
    /**
     * Instance of QUI\CmsImport\Import that is orchestrating the current import process
     *
     * @var ?Import
     */
    protected null | Import $Import = null;

    /**
     * The main QUIQQER Import console tool
     *
     * @var Console
     */
    protected Console $ImportConsole;

    /**
     * ImportProviderInterface constructor.
     *
     * @param Console $ImportConsole - The main QUIQQER Import console tool
     */
    public function __construct(Console $ImportConsole)
    {
        $this->ImportConsole = $ImportConsole;
    }

    /**
     * Write an info msg
     *
     * @param string $msg
     * @return void
     */
    public function writeInfo(string $msg): void
    {
        $this->ImportConsole->writeInfo($msg);
    }

    /**
     * Write a warning msg
     *
     * @param string $msg
     * @return void
     */
    public function writeWarning(string $msg): void
    {
        $this->ImportConsole->writeWarning($msg);
    }

    /**
     * Write an error msg
     *
     * @param string $msg
     * @return void
     */
    public function writeError(string $msg): void
    {
        $this->ImportConsole->writeError($msg);
    }

    /**
     * Output and return a user prompt
     *
     * @param string $msg
     * @param string|null $defaultValue (optional)
     * @return string
     */
    public function writePrompt(string $msg, null | string $defaultValue = null): string
    {
        return $this->ImportConsole->writePrompt($msg, $defaultValue);
    }

    /**
     * Write a header
     *
     * @param string $title
     * @return void
     */
    public function writeHeader(string $title): void
    {
        $this->ImportConsole->writeHeader($title);
    }

    /**
     * Set instance of QUI\CmsImport\Import that is orchestrating the current import process
     *
     * @param Import $Import
     * @return void
     */
    public function setImport(Import $Import): void
    {
        $this->Import = $Import;
    }

    /**
     * Get instance of QUI\CmsImport\Import that is orchestrating the current import process
     *
     * @return null|Import
     */
    public function getImport(): null | Import
    {
        return $this->Import;
    }

    /**
     * Get all import features this Provider provides
     *
     * @return array
     */
    public function getImportFeatures(): array
    {
        return [
            'importProjects' => true,
            'importSites' => true,
            'importTags' => true,
            'importMedia' => true,
            'importUsers' => true,
            'importGroups' => true,
            'importSystemConfig' => true,
            'importTranslations' => true,
            'importPermissions' => true
        ];
    }

    /**
     * These methods are executed after the import process finished. This allows the ImportProvider to do some
     * own cleanup stuff and/or other features.
     *
     * @return void
     */
    public function onImportFinished(): void
    {
        // nothing
    }

    /**
     * Get a special placeholder for a QUIQQER Image URL that is automatically parsed
     * to a QUIQQER Media Item URL on import.
     *
     * @param string $entityId
     * @return string
     */
    protected function getMediaUrlPlaceholder(string $entityId): string
    {
        return '[' . $this->Import::MEDIA_URL_BY_ENTITY_ID_PREFIX . $entityId . ']';
    }

    /**
     * Get a special placeholder for a QUIQQER Site URL that is automatically parsed
     * to a real QUIQQER Site URL on import.
     *
     * @param string $entityId
     * @return string
     */
    protected function getSiteUrlPlaceholder(string $entityId): string
    {
        return '[' . $this->Import::SITE_URL_BY_ENTITY_ID_PREFIX . $entityId . ']';
    }
}
