<?php

namespace QUI\CmsImport\Entities;

use QUI;

/**
 * Class ImportSite
 *
 * Represents a QUIQQER site that is imported
 */
class ImportSite extends AbstractImportEntity implements CustomQuiqqerIdInterface
{
    /**
     * Collection of tag titles associated with the Site
     *
     * @var array
     */
    protected array $tags = [];

    /**
     * Collection of tag group titles and their associated tag titles
     *
     * @var array
     */
    protected array $tagGroups = [];

    /**
     * List of language links
     *
     * @var array
     */
    protected array $languageLinks = [];

    /**
     * @var string|int
     */
    protected string | int $projectIdentifier;

    /**
     * @var string
     */
    protected string $name;

    /**
     * @var string
     */
    protected string $lang;

    /**
     * @var bool
     */
    protected bool $isFlaggedForReview = false;

    /**
     * @var ?int
     */
    protected ?int $quiqqerId = null;

    protected ?string $originalUrlPath = null;

    /**
     * ImportProject constructor.
     *
     * @param string|int $identifier - Unique identifier for this ImportSite
     * @param int|string $projectIdentifier - The name of the project this Site belongs to
     * @param string $name - Name of the Site (this is the part that is seen in the URL)
     * @param string $lang - Language of the site
     * @param array $attributes (optional) - Additional Site attributes
     */
    public function __construct(
        int | string $identifier,
        int | string $projectIdentifier,
        string $lang,
        string $name,
        array $attributes = []
    ) {
        $this->projectIdentifier = $projectIdentifier;
        $this->name = $name;
        $this->lang = $lang;

        // Default values
        $this->setAttributes([
            'type' => 'standard'
        ]);

        $this->setAttributes(array_merge($attributes, [
            'name' => $this->name
        ]));

        parent::__construct($identifier);
    }

    /**
     * Set QUIQQER ID
     *
     * @param int $id
     * @return void
     */
    public function setQuiqqerId(int $id): void
    {
        $this->quiqqerId = $id;
    }

    /**
     * Return QUIQQER ID
     *
     * @return int|null
     */
    public function getQuiqqerId(): ?int
    {
        return $this->quiqqerId;
    }

    /**
     * @return string|int
     */
    public function getProjectIdentifier(): int | string
    {
        return $this->projectIdentifier;
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * @return string
     */
    public function getLang(): string
    {
        return $this->lang;
    }

    /**
     * @param string $title - Simple text only
     */
    public function setTitle(string $title): void
    {
        $this->setAttribute('title', $title);
    }

    /**
     * @param string $short - Simple text only
     */
    public function setShort(string $short): void
    {
        $this->setAttribute('short', $short);
    }

    /**
     * @param string $content - Text which may contain HTML
     */
    public function setContent(string $content): void
    {
        $this->setAttribute('content', $content);
    }

    /**
     * @param string $siteType
     */
    public function setSiteType(string $siteType): void
    {
        $this->setAttribute('type', $siteType);
    }

    /**
     * Add a tag group that is associated with this Site
     *
     * @param string $tagGroupIdentifier
     * @return void
     */
    public function addTagGroup(string $tagGroupIdentifier): void
    {
        $this->tagGroups[] = $tagGroupIdentifier;
    }

    /**
     * Add a tag that is associated with this Site
     *
     * @param string $tagIdentifier
     * @return void
     */
    public function addTag(string $tagIdentifier): void
    {
        $this->tags[] = $tagIdentifier;
    }

    /**
     * @return array
     */
    public function getTags(): array
    {
        return array_values(array_unique($this->tags));
    }

    /**
     * @return array
     */
    public function getTagGroups(): array
    {
        return array_values(array_unique($this->tagGroups));
    }

    /**
     * Set a language link to a Site in another language
     *
     * @param string $lang
     * @param int $siteIdentifier - ImportSite identifier
     */
    public function setLanguageLink(string $lang, int $siteIdentifier): void
    {
        $this->languageLinks[$lang] = $siteIdentifier;
    }

    /**
     * @return array
     */
    public function getLanguageLinks(): array
    {
        return $this->languageLinks;
    }

    /**
     * Get the import section the ImportEntity belongs to
     *
     * @return string
     */
    public function getImportSection(): string
    {
        return QUI\CmsImport\Import::IMPORT_SECTION_SITES;
    }

    /**
     * @return string|null
     */
    public function getOriginalUrlPath(): ?string
    {
        return $this->originalUrlPath;
    }

    /**
     * @param string|null $originalUrlPath
     */
    public function setOriginalUrlPath(?string $originalUrlPath): void
    {
        $this->originalUrlPath = $originalUrlPath;
    }
}
