<?php

namespace QUI\CmsImport\Entities;

use QUI;

/**
 * Class ImportUser
 *
 * Represents a QUIQQER user that is imported
 */
class ImportUser extends AbstractImportEntity implements CustomQuiqqerIdInterface
{
    /**
     * @var string
     */
    protected string $username;

    /**
     * @var ?int
     */
    protected ?int $quiqqerId = null;

    /**
     * @var int[]
     */
    protected array $quiqqerGroupIds = [];

    /**
     * @var int|string[] - Group identifiers
     */
    protected int | array $groups = [];

    /**
     * @var ?string
     */
    protected ?string $passwordHash = null;

    /**
     * SuperUser flag
     *
     * @var bool
     */
    protected bool $isSU = false;

    /**
     * Administrator flag - administrators have QUIQQER backend access
     *
     * @var bool
     */
    protected bool $isAdmin = false;

    /**
     * Collection of user addresses
     *
     * @var array
     */
    protected array $addresses = [];

    /**
     * @var string|false
     */
    protected string | false $image = false;

    /**
     * ImportUser constructor.
     *
     * @param string|int $identifier - ImportGroup identifier
     * @param string $username - Username
     * @param array $attributes (optional) - Additional attributes
     */
    public function __construct(int | string $identifier, string $username, array $attributes = [])
    {
        $this->username = $username;

        $this->setAttributes($attributes);
        parent::__construct($identifier);
    }

    /**
     * @return int|string
     */
    public function getIdentifier(): int | string
    {
        return $this->identifier;
    }

    /**
     * @return string
     */
    public function getUsername(): string
    {
        return $this->username;
    }

    /**
     * Set QUIQQER ID
     *
     * @param int $id
     * @return void
     */
    public function setQuiqqerId(int $id): void
    {
        $this->quiqqerId = $id;
    }

    /**
     * Return QUIQQER ID
     *
     * @return int|null
     */
    public function getQuiqqerId(): ?int
    {
        return $this->quiqqerId;
    }

    /**
     * @return int[]
     */
    public function getQuiqqerGroupIds(): array
    {
        return $this->quiqqerGroupIds;
    }

    /**
     * Set IDs of existing QUIQQER groups
     *
     * @param int[] $quiqqerGroupIds
     */
    public function setQuiqqerGroupIds(array $quiqqerGroupIds): void
    {
        $this->quiqqerGroupIds = $quiqqerGroupIds;
    }

    /**
     * Set groups to the user
     *
     * @param array $groups - An array of unique ImportGroup identifiers
     * @return void
     */
    public function setGroups(array $groups): void
    {
        $this->groups = $groups;
    }

    /**
     * @return int|string[]
     */
    public function getGroups(): array | int
    {
        return $this->groups;
    }

    /**
     * @return string|null
     */
    public function getPasswordHash(): ?string
    {
        return $this->passwordHash;
    }

    /**
     * @param string $passwordHash
     */
    public function setPasswordHash(string $passwordHash): void
    {
        $this->passwordHash = $passwordHash;
    }

    /**
     * @return bool
     */
    public function isSU(): bool
    {
        return $this->isSU;
    }

    /**
     * @param bool $isSU
     */
    public function setIsSU(bool $isSU): void
    {
        $this->isSU = $isSU;
    }

    /**
     * @return bool
     */
    public function isAdmin(): bool
    {
        return $this->isAdmin;
    }

    /**
     * @param bool $isAdmin
     */
    public function setIsAdmin(bool $isAdmin): void
    {
        $this->isAdmin = $isAdmin;
    }

    /**
     * @param array $address
     *
     * Available $address keys:
     * [
     *      'default' => true / false
     *      'salutation',
     *      'firstname',
     *      'lastname',
     *      'mail',
     *      'company',
     *      'street_no',
     *      'zip',
     *      'city',
     *      'country',
     *      'phone' => [123, 312],
     *      'mobile' => [123, 312],
     *      'fax' => [123, 321]
     * ]
     */
    public function addAddress(array $address): void
    {
        $this->addresses[] = $address;
    }

    /**
     * @return array
     */
    public function getAddresses(): array
    {
        return $this->addresses;
    }

    /**
     * Get the import section the ImportEntity belongs to
     *
     * @return string
     */
    public function getImportSection(): string
    {
        return QUI\CmsImport\Import::IMPORT_SECTION_USERS;
    }

    /**
     * Get filepath to user image
     *
     * @return string
     */
    public function getImage(): string
    {
        return $this->image;
    }

    /**
     * Set filepath to user image
     *
     * @param string $image
     */
    public function setImage(string $image): void
    {
        $this->image = $image;
    }
}
