<?php

namespace QUI\CmsImport;

use QUI\CmsImport\Entities\ImportGroup;
use QUI\CmsImport\Entities\ImportMediaItem;
use QUI\CmsImport\Entities\ImportPermission;
use QUI\CmsImport\Entities\ImportProject;
use QUI\CmsImport\Entities\ImportSite;
use QUI\CmsImport\Entities\ImportTag;
use QUI\CmsImport\Entities\ImportTagGroup;
use QUI\CmsImport\Entities\ImportTranslation;
use QUI\CmsImport\Entities\ImportUser;
use QUI\CmsImport\MetaEntities\MetaHierarchy;
use QUI\CmsImport\MetaEntities\ProjectList;
use QUI\CmsImport\MetaEntities\SiteHierarchy;
use QUI\CmsImport\MetaEntities\MetaList;

/**
 * Interface ImportProviderInterface
 *
 * Interface for all providers that collect and provide data for a QUIQQER CMS Import
 */
interface ImportProviderInterface
{
    /**
     * ImportProviderInterface constructor.
     *
     * @param Console $ImportConsole - The main QUIQQER Import console tool
     */
    public function __construct(Console $ImportConsole);

    /**
     * Get title of this Import Provider
     *
     * @return string
     */
    public function getTitle(): string;

    /**
     * Get description of this Import Provider
     *
     * @return string
     */
    public function getDescription(): string;

    /**
     * Get all QUIQQER projects that should be imported
     *
     * @return ProjectList
     */
    public function getProjectList(): ProjectList;

    /**
     * Get an ImportProject
     *
     * @param int|string $projectIdentifier - Unique ImportProject identifier
     * @return ImportProject
     */
    public function getProject(int | string $projectIdentifier): ImportProject;

    /**
     * Get a site that should be imported
     *
     * @param int|string $siteIdentifier - A unique ImportSite identifier
     * @param string $projectIdentifier - A unique ImportProject identifier
     * @param string $lang - Project lang
     * @return ImportSite
     */
    public function getSite(int | string $siteIdentifier, string $projectIdentifier, string $lang): ImportSite;

    /**
     * Get the complete hierarchical site structure (structure only, not actual site data!)
     *
     * @param string $projectIdentifier - A unique ImportProject identifier
     * @param string $lang - Project lang
     * @return SiteHierarchy
     */
    public function getSiteHierarchy(string $projectIdentifier, string $lang): SiteHierarchy;

    /**
     * Get the complete hierarchical media item structure (folders, files, images)
     *
     * @param string $projectIdentifier
     * @return MetaHierarchy
     */
    public function getMediaHierarchy(string $projectIdentifier): MetaHierarchy;

    /**
     * Get an ImportMediaItem
     *
     * @param int|string $mediaItemIdentifier
     * @param string $projectIdentifier
     * @return ImportMediaItem
     */
    public function getMediaItem(int | string $mediaItemIdentifier, string $projectIdentifier): ImportMediaItem;

    /**
     * Get complete group hierarchy for QUIQQER group structure
     *
     * @return MetaHierarchy
     */
    public function getGroupHierarchy(): MetaHierarchy;

    /**
     * Get ImportGroup by identifier
     *
     * @param int|string $groupIdentifier
     * @return ImportGroup
     */
    public function getGroup(int | string $groupIdentifier): ImportGroup;

    /**
     * Get complete list of QUIQQER users (meta-list, not actual User objects)
     *
     * @return MetaList
     */
    public function getUserList(): MetaList;

    /**
     * Get ImportUser by identifier
     *
     * @param int|string $userIdentifier
     * @return ImportUser
     */
    public function getUser(int | string $userIdentifier): ImportUser;

    /**
     * Get complete TagGroupHierarchy for a project
     *
     * @param string $projectIdentifier - A unique ImportProject identifier
     * @param string $lang - Project lang
     * @return MetaHierarchy
     */
    public function getTagGroupHierarchy(string $projectIdentifier, string $lang): MetaHierarchy;

    /**
     * Get an ImportTagGroup
     *
     * @param int|string $identifier - Unique ImportTagGroup identifier
     * @param string $projectIdentifier - A unique ImportProject identifier
     * @param string $lang - Project lang
     * @return ImportTagGroup
     */
    public function getTagGroup(int | string $identifier, string $projectIdentifier, string $lang): ImportTagGroup;

    /**
     * Get complete list of ImportTag identifiers
     *
     * @param string $projectIdentifier - A unique ImportProject identifier
     * @param string $lang - Project lang
     * @return MetaList
     */
    public function getTagList(string $projectIdentifier, string $lang): MetaList;

    /**
     * Get an ImportTag
     *
     * @param int|string $identifier - Unique ImportTagGroup identifier
     * @param string $projectIdentifier - A unique ImportProject identifier
     * @param string $lang - Project lang
     * @return ImportTag
     */
    public function getTag(int | string $identifier, string $projectIdentifier, string $lang): ImportTag;

    /**
     * Get QUIQQER system configuration (as associative array)
     *
     * These are the settings for etc/conf.ini.php
     *
     * @return array
     */
    public function getSystemConfig(): array;

    /**
     * Get all translations for a project
     *
     * @param int|string $projectIdentifier - Unique ImportProject identifier
     * @return MetaList
     */
    public function getTranslationList(int | string $projectIdentifier): MetaList;

    /**
     * Get an ImportTranslation
     *
     * @param int|string $translationIdentifier
     * @param int|string $projectIdentifier
     * @return ImportTranslation
     */
    public function getTranslation(
        int | string $translationIdentifier,
        int | string $projectIdentifier
    ): ImportTranslation;

    /**
     * Get all permissions that shall be imported
     *
     * @return MetaList
     */
    public function getPermissionList(): MetaList;

    /**
     * Get an ImportPermission
     *
     * @param int|string $permissionIdentifier
     * @return ImportPermission
     */
    public function getPermission(int | string $permissionIdentifier): ImportPermission;

    /**
     * The purpose of this method is to prompt the user for necessary configuration data (like paths, DB credentials...)
     * regarding the system the import data is pulled from.
     *
     * If the ImportProvider collects its configuration in a different manner (e.g. a config file) this method
     * does not have to actually do anything.
     *
     * @return void
     */
    public function promptForConfig(): void;

    /**
     * Set instance of QUI\CmsImport\Import that is orchestrating the current import process
     *
     * @param Import $Import
     * @return void
     */
    public function setImport(Import $Import): void;

    /**
     * Get instance of QUI\CmsImport\Import that is orchestrating the current import process
     *
     * @return null|Import
     */
    public function getImport(): null|Import;

    /**
     * Get all import features this Provider provides
     *
     * @return array
     */
    public function getImportFeatures(): array;

    /**
     * These methods are executed after the import process finished. This allows the ImportProvider to do some
     * own cleanup stuff and/or other features.
     *
     * @return void
     */
    public function onImportFinished(): void;
}
