<?php

namespace QUI\CmsImport\MetaEntities;

use Generator;

use function is_float;
use function is_int;
use function strcmp;

/**
 * Class MetaItemList
 *
 * A simple list of MetaEntity objects
 */
class MetaList implements ChildrenInterface
{
    /**
     * @var MetaEntity[]
     */
    protected array $children = [];

    /**
     * Add a child to the list
     *
     * @param MetaEntity $Item
     * @return void
     */
    public function addChild(MetaEntity $Item): void
    {
        $this->children[] = $Item;
    }

    /**
     * Get a child from the list
     *
     * @param int|string $id
     * @return bool|MetaEntity
     */
    public function getChild(int | string $id): bool | MetaEntity
    {
        foreach ($this->walkChildren() as $Child) {
            if ($Child->getId() == $id) {
                return $Child;
            }
        }

        return false;
    }

    /**
     * @return MetaEntity[]
     */
    public function getChildren(): array
    {
        return $this->children;
    }

    /**
     * Walk the list of children
     *
     * @return Generator
     */
    public function walkChildren(): Generator
    {
        foreach ($this->children as $k => $Child) {
            yield $k => $Child;
        }
    }

    /**
     * @return bool
     */
    public function hasChildren(): bool
    {
        return !empty($this->children);
    }

    /**
     * @param string $attribute - Name of the attribute that is sorted by
     * @param string $type - Type of sort (see ChildrenInterface::SORT_TYPE_*)
     * @return void
     */
    public function sortChildren(string $attribute, string $type = self::SORT_TYPE_ASC): void
    {
        usort($this->children, function ($ChildA, $ChildB) use ($type) {
            $valA = $ChildA->getAttribute($type);
            $valB = $ChildB->getAttribute($type);

            // Numeric comparison
            if ((is_int($valA) || is_float($valA)) && (is_int($valB) || is_float($valB))) {
                if ($valA < $valB) {
                    return match ($type) {
                        self::SORT_TYPE_DESC => 1,
                        default => -1,
                    };
                }

                if ($valA > $valB) {
                    return match ($type) {
                        self::SORT_TYPE_DESC => -1,
                        default => 1,
                    };
                }

                return 0;
            }

            // String comparison
            return match ($type) {
                self::SORT_TYPE_DESC => strcmp($valA, $valB) * (-1),
                default => strcmp($valA, $valB),
            };
        });
    }

    /**
     * Sort children by a custom sort function
     *
     * @param callable $sortFunc
     * @return void
     */
    public function sortChildrenCustom(callable $sortFunc): void
    {
        usort($this->children, $sortFunc);
    }
}
